@@ -26,26 +26,25 @@
   EFFECT(channels)
   EFFECT(compand)
   EFFECT(contrast)
-  EFFECT(crop)
   EFFECT(dcshift)
   EFFECT(deemph)
   EFFECT(delay)
   EFFECT(dft_filter) /* abstract */
   EFFECT(dither)
   EFFECT(divide)
+  EFFECT(downsample)
   EFFECT(earwax)
   EFFECT(echo)
   EFFECT(echos)
   EFFECT(equalizer)
   EFFECT(fade)
-  EFFECT(filter)
   EFFECT(fir)
   EFFECT(firfit)
   EFFECT(flanger)
   EFFECT(gain)
   EFFECT(highpass)
+  EFFECT(hilbert)
   EFFECT(input)
-  EFFECT(key)
 #ifdef HAVE_LADSPA_H
   EFFECT(ladspa)
 #endif
@@ -60,15 +59,11 @@
   EFFECT(output)
   EFFECT(overdrive)
   EFFECT(pad)
-  EFFECT(pan)
   EFFECT(phaser)
   EFFECT(pitch)
-  EFFECT(polyphase)
-  EFFECT(rabbit)
   EFFECT(rate)
   EFFECT(remix)
   EFFECT(repeat)
-  EFFECT(resample)
   EFFECT(reverb)
   EFFECT(reverse)
   EFFECT(riaa)
@@ -91,5 +86,6 @@
   EFFECT(treble)
   EFFECT(tremolo)
   EFFECT(trim)
+  EFFECT(upsample)
   EFFECT(vad)
   EFFECT(vol)
