@@ -37,7 +37,7 @@
   #include <magic.h>
 #endif
 
-#define AUTO_DETECT_SIZE 256
+#define AUTO_DETECT_SIZE 4096
 
 static char const * auto_detect_format(sox_format_t * ft, char const * ext)
 {
@@ -112,40 +112,46 @@ static char const * auto_detect_format(sox_format_t * ft, char const * ext)
   return NULL;
 }
 
-sox_encodings_info_t const sox_encodings_info[] = {
-  {0         , "n/a"          , "Unknown or not applicable"},
-  {0         , "Signed PCM"   , "Signed Integer PCM"},
-  {0         , "Unsigned PCM" , "Unsigned Integer PCM"},
-  {0         , "F.P. PCM"     , "Floating Point PCM"},
-  {0         , "F.P. PCM"     , "Floating Point (text) PCM"},
-  {0         , "FLAC"         , "FLAC"},
-  {0         , "HCOM"         , "HCOM"},
-  {0         , "WavPack"      , "WavPack"},
-  {0         , "F.P. WavPack" , "Floating Point WavPack"},
-  {SOX_LOSSY1, "u-law"        , "u-law"},
-  {SOX_LOSSY1, "A-law"        , "A-law"},
-  {SOX_LOSSY1, "G.721 ADPCM"  , "G.721 ADPCM"},
-  {SOX_LOSSY1, "G.723 ADPCM"  , "G.723 ADPCM"},
-  {SOX_LOSSY1, "CL ADPCM (8)" , "CL ADPCM (from 8-bit)"},
-  {SOX_LOSSY1, "CL ADPCM (16)", "CL ADPCM (from 16-bit)"},
-  {SOX_LOSSY1, "MS ADPCM"     , "MS ADPCM"},
-  {SOX_LOSSY1, "IMA ADPCM"    , "IMA ADPCM"},
-  {SOX_LOSSY1, "OKI ADPCM"    , "OKI ADPCM"},
-  {SOX_LOSSY1, "DPCM"         , "DPCM"},
-  {0         , "DWVW"         , "DWVW"},
-  {0         , "DWVWN"        , "DWVWN"},
-  {SOX_LOSSY2, "GSM"          , "GSM"},
-  {SOX_LOSSY2, "MPEG audio"   , "MPEG audio (layer I, II or III)"},
-  {SOX_LOSSY2, "Vorbis"       , "Vorbis"},
-  {SOX_LOSSY2, "AMR-WB"       , "AMR-WB"},
-  {SOX_LOSSY2, "AMR-NB"       , "AMR-NB"},
-  {SOX_LOSSY2, "CVSD"         , "CVSD"},
-  {SOX_LOSSY2, "LPC10"        , "LPC10"},
+static sox_encodings_info_t const s_sox_encodings_info[] = {
+  {sox_encodings_none  , "n/a"          , "Unknown or not applicable"},
+  {sox_encodings_none  , "Signed PCM"   , "Signed Integer PCM"},
+  {sox_encodings_none  , "Unsigned PCM" , "Unsigned Integer PCM"},
+  {sox_encodings_none  , "F.P. PCM"     , "Floating Point PCM"},
+  {sox_encodings_none  , "F.P. PCM"     , "Floating Point (text) PCM"},
+  {sox_encodings_none  , "FLAC"         , "FLAC"},
+  {sox_encodings_none  , "HCOM"         , "HCOM"},
+  {sox_encodings_none  , "WavPack"      , "WavPack"},
+  {sox_encodings_none  , "F.P. WavPack" , "Floating Point WavPack"},
+  {sox_encodings_lossy1, "u-law"        , "u-law"},
+  {sox_encodings_lossy1, "A-law"        , "A-law"},
+  {sox_encodings_lossy1, "G.721 ADPCM"  , "G.721 ADPCM"},
+  {sox_encodings_lossy1, "G.723 ADPCM"  , "G.723 ADPCM"},
+  {sox_encodings_lossy1, "CL ADPCM (8)" , "CL ADPCM (from 8-bit)"},
+  {sox_encodings_lossy1, "CL ADPCM (16)", "CL ADPCM (from 16-bit)"},
+  {sox_encodings_lossy1, "MS ADPCM"     , "MS ADPCM"},
+  {sox_encodings_lossy1, "IMA ADPCM"    , "IMA ADPCM"},
+  {sox_encodings_lossy1, "OKI ADPCM"    , "OKI ADPCM"},
+  {sox_encodings_lossy1, "DPCM"         , "DPCM"},
+  {sox_encodings_none  , "DWVW"         , "DWVW"},
+  {sox_encodings_none  , "DWVWN"        , "DWVWN"},
+  {sox_encodings_lossy2, "GSM"          , "GSM"},
+  {sox_encodings_lossy2, "MPEG audio"   , "MPEG audio (layer I, II or III)"},
+  {sox_encodings_lossy2, "Vorbis"       , "Vorbis"},
+  {sox_encodings_lossy2, "AMR-WB"       , "AMR-WB"},
+  {sox_encodings_lossy2, "AMR-NB"       , "AMR-NB"},
+  {sox_encodings_lossy2, "CVSD"         , "CVSD"},
+  {sox_encodings_lossy2, "LPC10"        , "LPC10"},
 };
 
-assert_static(array_length(sox_encodings_info) == SOX_ENCODINGS,
+assert_static(array_length(s_sox_encodings_info) == SOX_ENCODINGS,
     SIZE_MISMATCH_BETWEEN_sox_encoding_t_AND_sox_encodings_info);
 
+sox_encodings_info_t const *
+sox_get_encodings_info(void)
+{
+    return s_sox_encodings_info;
+}
+
 unsigned sox_precision(sox_encoding_t encoding, unsigned bits_per_sample)
 {
   switch (encoding) {
@@ -191,9 +197,9 @@ unsigned sox_precision(sox_encoding_t encoding, unsigned bits_per_sample)
 
 void sox_init_encodinginfo(sox_encodinginfo_t * e)
 {
-  e->reverse_bytes = SOX_OPTION_DEFAULT;
-  e->reverse_nibbles = SOX_OPTION_DEFAULT;
-  e->reverse_bits = SOX_OPTION_DEFAULT;
+  e->reverse_bytes = sox_option_default;
+  e->reverse_nibbles = sox_option_default;
+  e->reverse_bits = sox_option_default;
   e->compression = HUGE_VAL;
 }
 
@@ -289,7 +295,7 @@ static void set_endiannesses(sox_format_t * ft)
   if (ft->encoding.opposite_endian)
     ft->encoding.reverse_bytes = (ft->handler.flags & SOX_FILE_ENDIAN)?
       !(ft->handler.flags & SOX_FILE_ENDBIG) != MACHINE_IS_BIGENDIAN : sox_true;
-  else if (ft->encoding.reverse_bytes == SOX_OPTION_DEFAULT)
+  else if (ft->encoding.reverse_bytes == sox_option_default)
     ft->encoding.reverse_bytes = (ft->handler.flags & SOX_FILE_ENDIAN)?
       !(ft->handler.flags & SOX_FILE_ENDBIG) == MACHINE_IS_BIGENDIAN : sox_false;
 
@@ -300,15 +306,15 @@ static void set_endiannesses(sox_format_t * ft)
     if (ft->encoding.reverse_bytes == (sox_option_t)
         (!(ft->handler.flags & SOX_FILE_ENDBIG) != MACHINE_IS_BIGENDIAN))
       lsx_report("`%s': overriding file-type byte-order", ft->filename);
-  } else if (ft->encoding.reverse_bytes == sox_true)
+  } else if (ft->encoding.reverse_bytes == sox_option_yes)
     lsx_report("`%s': overriding machine byte-order", ft->filename);
 
-  if (ft->encoding.reverse_bits == SOX_OPTION_DEFAULT)
+  if (ft->encoding.reverse_bits == sox_option_default)
     ft->encoding.reverse_bits = !!(ft->handler.flags & SOX_FILE_BIT_REV);
   else if (ft->encoding.reverse_bits == !(ft->handler.flags & SOX_FILE_BIT_REV))
       lsx_report("`%s': overriding file-type bit-order", ft->filename);
 
-  if (ft->encoding.reverse_nibbles == SOX_OPTION_DEFAULT)
+  if (ft->encoding.reverse_nibbles == sox_option_default)
     ft->encoding.reverse_nibbles = !!(ft->handler.flags & SOX_FILE_NIB_REV);
   else
     if (ft->encoding.reverse_nibbles == !(ft->handler.flags & SOX_FILE_NIB_REV))
@@ -322,7 +328,7 @@ static sox_bool is_seekable(sox_format_t const * ft)
   assert(ft);
   if (!ft->fp)
     return sox_false;
-  fstat(fileno(ft->fp), &st);
+  fstat(fileno((FILE*)ft->fp), &st);
   return ((st.st_mode & S_IFMT) == S_IFREG);
 }
 
@@ -366,7 +372,10 @@ static FILE * xfopen(char const * identifier, char const * mode, lsx_io_type * i
   if (*identifier == '|') {
     FILE * f = NULL;
 #ifdef HAVE_POPEN
-    f = popen(identifier + 1, "r");
+#ifndef POPEN_MODE
+#define POPEN_MODE "r"
+#endif
+    f = popen(identifier + 1, POPEN_MODE);
     *io_type = lsx_io_pipe;
 #else
     lsx_fail("this build of SoX cannot open pipes");
@@ -379,7 +388,7 @@ static FILE * xfopen(char const * identifier, char const * mode, lsx_io_type * i
     char const * const command_format = "wget --no-check-certificate -q -O- \"%s\"";
     char * command = lsx_malloc(strlen(command_format) + strlen(identifier));
     sprintf(command, command_format, identifier);
-    f = popen(command, "r");
+    f = popen(command, POPEN_MODE);
     free(command);
     *io_type = lsx_io_url;
 #else
@@ -591,7 +600,7 @@ sox_bool sox_format_supports_encoding(
   unsigned i = 0, s;
   sox_encoding_t e;
 
-  assert(path);
+  assert(path || filetype);
   assert(encoding);
   if (!filetype)
     filetype = lsx_find_file_extension(path);
@@ -617,7 +626,7 @@ sox_bool sox_format_supports_encoding(
 
 static void set_output_format(sox_format_t * ft)
 {
-  sox_encoding_t e;
+  sox_encoding_t e = SOX_ENCODING_UNKNOWN;
   unsigned i, s;
   unsigned const * encodings = ft->handler.write_formats;
 #define enc_arg(T) (T)encodings[i++]
@@ -728,7 +737,7 @@ static void set_output_format(sox_format_t * ft)
     i = 0;
     while ((e = enc_arg(sox_encoding_t)))
       while ((s = enc_arg(unsigned)))
-        if (!(sox_encodings_info[e].flags & (SOX_LOSSY1 | SOX_LOSSY2)) &&
+        if (!(sox_encodings_info[e].flags & (sox_encodings_lossy1 | sox_encodings_lossy2)) &&
             sox_precision(e, s) >= ft->signal.precision && s < ft->encoding.bits_per_sample) {
           ft->encoding.encoding = e;
           ft->encoding.bits_per_sample = s;
@@ -762,7 +771,7 @@ static void set_output_format(sox_format_t * ft)
       ft->encoding.bits_per_sample = max_p_s;
     }
   }
-  ft->signal.precision = min(ft->signal.precision, sox_precision(ft->encoding.encoding, ft->encoding.bits_per_sample));
+  ft->signal.precision = sox_precision(ft->encoding.encoding, ft->encoding.bits_per_sample);
   #undef enc_arg
 }
 
@@ -1002,7 +1011,7 @@ int sox_close(sox_format_t * ft)
   return result;
 }
 
-int sox_seek(sox_format_t * ft, uint64_t offset, int whence)
+int sox_seek(sox_format_t * ft, sox_uint64_t offset, int whence)
 {
     /* FIXME: Implement SOX_SEEK_CUR and SOX_SEEK_END. */
     if (whence != SOX_SEEK_SET)
@@ -1150,7 +1159,28 @@ static sox_bool plugins_initted = sox_false;
   #define MAX_FORMATS (NSTATIC_FORMATS + MAX_DYNAMIC_FORMATS)
   #define MAX_FORMATS_1 (MAX_FORMATS + 1)
   #define MAX_NAME_LEN (size_t)1024 /* FIXME: Use vasprintf */
+#else
+  #define MAX_FORMATS_1
+#endif
 
+#define FORMAT(f) extern sox_format_handler_t const * lsx_##f##_format_fn(void);
+#include "formats.h"
+#undef FORMAT
+
+static sox_format_tab_t s_sox_format_fns[MAX_FORMATS_1] = {
+  #define FORMAT(f) {NULL, lsx_##f##_format_fn},
+  #include "formats.h"
+  #undef FORMAT
+  {NULL, NULL}
+};
+
+const sox_format_tab_t *
+sox_get_format_fns(void)
+{
+    return s_sox_format_fns;
+}
+
+#ifdef HAVE_LIBLTDL /* Plugin format handlers */
   static unsigned nformats = NSTATIC_FORMATS;
 
   static int init_format(const char *file, lt_ptr data)
@@ -1176,27 +1206,14 @@ static sox_bool plugins_initted = sox_false;
             lsx_warn("too many plugin formats");
             return -1;
           }
-          sox_format_fns[nformats++].fn = ltptr.fn;
+          s_sox_format_fns[nformats++].fn = ltptr.fn;
         }
       }
     }
     return 0;
   }
-#else
-  #define MAX_FORMATS_1
 #endif
 
-#define FORMAT(f) extern sox_format_handler_t const * lsx_##f##_format_fn(void);
-#include "formats.h"
-#undef FORMAT
-
-sox_format_tab_t sox_format_fns[MAX_FORMATS_1] = {
-  #define FORMAT(f) {NULL, lsx_##f##_format_fn},
-  #include "formats.h"
-  #undef FORMAT
-  {NULL, NULL}
-};
-
 int sox_format_init(void) /* Find & load format handlers.  */
 {
   if (plugins_initted)
@@ -1222,6 +1239,8 @@ void sox_format_quit(void) /* Cleanup things.  */
   int ret;
   if (plugins_initted && (ret = lt_dlexit()) != 0)
     lsx_fail("lt_dlexit failed with %d error(s): %s", ret, lt_dlerror());
+  plugins_initted = sox_false;
+  nformats = NSTATIC_FORMATS;
 #endif
 }
 
@@ -1245,8 +1264,8 @@ sox_format_handler_t const * sox_find_format(char const * name0, sox_bool no_dev
     char * pos = strchr(name, ';');
     if (pos) /* Use only the 1st clause of a mime string */
       *pos = '\0';
-    for (f = 0; sox_format_fns[f].fn; ++f) {
-      sox_format_handler_t const * handler = sox_format_fns[f].fn();
+    for (f = 0; s_sox_format_fns[f].fn; ++f) {
+      sox_format_handler_t const * handler = s_sox_format_fns[f].fn();
 
       if (!(no_dev && (handler->flags & SOX_FILE_DEVICE)))
         for (n = 0; handler->names[n]; ++n)
