@@ -392,8 +392,11 @@ static int findChunk(sox_format_t * ft, const char *Label, uint32_t *len)
         if (strncmp(Label, magic, (size_t)4) == 0)
             break; /* Found the given chunk */
 
+	/* Chunks are required to be word aligned. */
+	if ((*len) % 2) (*len)++;
+
         /* skip to next chunk */
-        if (*len == 0 || lsx_seeki(ft, (off_t)(*len), SEEK_CUR) != SOX_SUCCESS)
+        if (*len > 0 && lsx_seeki(ft, (off_t)(*len), SEEK_CUR) != SOX_SUCCESS)
         {
             lsx_fail_errno(ft,SOX_EHDR,
                           "WAV chunk appears to have invalid size %d.", *len);
@@ -406,7 +409,7 @@ static int findChunk(sox_format_t * ft, const char *Label, uint32_t *len)
 
 static int wavfail(sox_format_t * ft, const char *format)
 {
-    lsx_fail_errno(ft, SOX_EHDR, "Unhandled WAV file encoding (%s).\nTry overriding the encoding: e.g. for an MP3 WAV, `-t mp3'", format);
+    lsx_fail_errno(ft, SOX_EHDR, "WAV file encoding `%s' is not supported", format);
     return SOX_EOF;
 }
 
@@ -882,7 +885,7 @@ static int startread(sox_format_t * ft)
     }
 
     /* Horrible way to find Cool Edit marker points. Taken from Quake source*/
-    ft->oob.loops[0].start = ~0u;
+    ft->oob.loops[0].start = SOX_IGNORE_LENGTH;
     if(ft->seekable){
         /*Got this from the quake source.  I think it 32bit aligns the chunks
          * doubt any machine writing Cool Edit Chunks writes them at an odd
@@ -1190,6 +1193,7 @@ PCM formats then go straight to the data chunk:
 36 - 39    'data'
 40 - 43     dwDataLength   length of data chunk minus 8 byte header
 44 - (dwDataLength + 43)   the data
+(+ a padding byte if dwDataLength is odd)
 
 non-PCM formats must write an extended format chunk and a fact chunk:
 
@@ -1201,6 +1205,7 @@ ULAW, ALAW formats:
 50 - 53    'data'
 54 - 57     dwDataLength  length of data chunk minus 8 byte header
 58 - (dwDataLength + 57)  the data
+(+ a padding byte if dwDataLength is odd)
 
 
 GSM6.10  format:
@@ -1211,8 +1216,8 @@ GSM6.10  format:
 48 - 51    dwSamplesWritten   actual number of samples written out
 52 - 55    'data'
 56 - 59     dwDataLength  length of data chunk minus 8 byte header
-60 - (dwDataLength + 59)  the data
-(+ a padding byte if dwDataLength is odd)
+60 - (dwDataLength + 59)  the data (including a padding byte, if necessary,
+                            so dwDataLength is always even)
 
 
 note that header contains (up to) 3 separate ways of describing the
@@ -1361,8 +1366,8 @@ static int wavwritehdr(sox_format_t * ft, int second_header)
     else if (wFormatTag != WAVE_FORMAT_PCM)
         wFmtSize += 2+wExtSize; /* plus ExtData */
 
-    wRiffLength = 4 + (8+wFmtSize) + (8+dwDataLength);
-    if (wFormatTag != WAVE_FORMAT_PCM) /* PCM omits the "fact" chunk */
+    wRiffLength = 4 + (8+wFmtSize) + (8+dwDataLength+dwDataLength%2);
+    if (isExtensible || wFormatTag != WAVE_FORMAT_PCM) /* PCM omits the "fact" chunk */
         wRiffLength += (8+dwFactSize);
 
     /* dwAvgBytesPerSec <-- this is BEFORE compression, isn't it? guess not. */
@@ -1393,12 +1398,25 @@ static int wavwritehdr(sox_format_t * ft, int second_header)
     lsx_writew(ft, wBitsPerSample); /* end info common to all fmts */
 
     if (isExtensible) {
+      uint32_t dwChannelMask=0;  /* unassigned speaker mapping by default */
       static unsigned char const guids[][14] = {
         "\x00\x00\x00\x00\x10\x00\x80\x00\x00\xAA\x00\x38\x9B\x71",  /* wav */
         "\x00\x00\x21\x07\xd3\x11\x86\x44\xc8\xc1\xca\x00\x00\x00"}; /* amb */
+
+      /* if not amb, assume most likely channel masks from number of channels; not
+       * ideal solution, but will make files playable in many/most situations
+       */
+      if (strcmp(ft->filetype, "amb")) {
+        if      (wChannels == 1) dwChannelMask = 0x4;     /* 1 channel (mono) = FC */
+        else if (wChannels == 2) dwChannelMask = 0x3;     /* 2 channels (stereo) = FL, FR */
+        else if (wChannels == 4) dwChannelMask = 0x33;    /* 4 channels (quad) = FL, FR, BL, BR */
+        else if (wChannels == 6) dwChannelMask = 0x3F;    /* 6 channels (5.1) = FL, FR, FC, LF, BL, BR */
+        else if (wChannels == 8) dwChannelMask = 0x63F;   /* 8 channels (7.1) = FL, FR, FC, LF, BL, BR, SL, SR */
+      }
+ 
       lsx_writew(ft, 22);
       lsx_writew(ft, wBitsPerSample); /* No padding in container */
-      lsx_writedw(ft, 0);             /* Speaker mapping not specified */
+      lsx_writedw(ft, dwChannelMask); /* Speaker mapping is something reasonable */
       lsx_writew(ft, wFormatTag);
       lsx_writebuf(ft, guids[!strcmp(ft->filetype, "amb")], (size_t)14);
     }
@@ -1518,6 +1536,12 @@ static int stopwrite(sox_format_t * ft)
             wavgsmstopwrite(ft);
             break;
         }
+
+        /* Add a pad byte if the number of data bytes is odd.
+           See wavwritehdr() above for the calculation. */
+        if (wav->formatTag != WAVE_FORMAT_GSM610)
+          lsx_padbytes(ft, (size_t)((wav->numSamples + wav->samplesPerBlock - 1)/wav->samplesPerBlock*wav->blockAlign) % 2);
+
         free(wav->packet);
         free(wav->samples);
         free(wav->lsx_ms_adpcm_i_coefs);
@@ -1599,7 +1623,7 @@ static int seek(sox_format_t * ft, uint64_t offset)
   if (ft->encoding.bits_per_sample & 7)
     lsx_fail_errno(ft, SOX_ENOTSUP, "seeking not supported with this encoding");
   else if (wav->formatTag == WAVE_FORMAT_GSM610) {
-    int new_offset, alignment;
+    int alignment;
     size_t gsmoff;
 
     /* rounding bytes to blockAlign so that we
@@ -1611,7 +1635,7 @@ static int seek(sox_format_t * ft, uint64_t offset)
     ft->sox_errno = lsx_seeki(ft, (off_t)(gsmoff + wav->dataStart), SEEK_SET);
     if (ft->sox_errno == SOX_SUCCESS) {
       /* offset is in samples */
-      new_offset = offset;
+      uint64_t new_offset = offset;
       alignment = offset % wav->samplesPerBlock;
       if (alignment != 0)
           new_offset += (wav->samplesPerBlock - alignment);
